"""Cliff command module for the Inline command"""

import io
import logging
import os
import re

from cliff.command import Command


class Inline(Command):
    """Inline all files in use/include statements."""

    log = logging.getLogger(__name__)
    outfile: io.TextIOWrapper
    filenames = []
    statement_regex = re.compile("(use|include) <[^>]+>")

    def get_parser(self, prog_name) -> object:
        """Set up and return the parser.

        :param prog_name: name of program

        """
        parser = super().get_parser(prog_name)
        parser.add_argument("filename", nargs="?")
        return parser

    def scan_file(self, filename) -> None:
        """Scan a file for include and use statements.

        :param filename: string name of file

        """
        basename = os.path.basename(filename)
        if basename not in self.filenames:
            self.log.debug(basename)
            self.filenames.append(basename)
            with open(filename, mode="r", encoding="utf-8") as infile:
                self.log.debug("opening %s...", filename)
                directory = os.path.dirname(filename)
                for line in infile.readlines():
                    if self.statement_regex.match(line.lstrip().rstrip()):
                        incl_file = os.path.join(
                            directory, line[line.index("<") + 1 : line.index(">")]
                        )
                        if os.path.isfile(incl_file):
                            file_path = os.path.abspath(incl_file)
                            self.scan_file(file_path)
                    else:
                        self.outfile.write(line.rstrip() + "\n")

    def take_action(self, parsed_args) -> None:
        """Perform action on file

        :param parsed_args: structure of parsed arguments
        :returns: None

        """
        self.log.debug("parsed_args: %s", parsed_args)
        try:
            infile = os.path.abspath(parsed_args.filename)
            if not os.path.exists(infile):
                self.log.error("No such file or directory.")
                return
            directory = os.path.dirname(infile)
            with open(
                os.path.join(directory, f"inline-{os.path.basename(infile)}"),
                "w",
                encoding="utf-8",
            ) as self.outfile:
                self.scan_file(infile)
        except (FileNotFoundError, TypeError) as ex:
            if isinstance(ex, FileNotFoundError):
                self.log.error("No such file or directory.")
            else:
                self.log.error("Please enter a filename.")



"""Cliff application class for Scadi."""

import sys
from pkg_resources import get_distribution

from cliff.app import App
from cliff.commandmanager import CommandManager


class ScadiApp(App):
    """Scadi cliff.App implementation"""

    def __init__(self) -> None:
        """Create a new instance of ScadiApp

        :returns: None

        """
        super().__init__(
            description="OpenSCAD Inliner",
            version=get_distribution("scadi").version,
            command_manager=CommandManager("cliff.scadi"),
            deferred_help=True,
        )

    def initialize_app(self, _) -> None:
        """Initialize cliff CommandManager app

        :returns: None

        """
        self.LOG.debug("initialize_app")

    def prepare_to_run_command(self, cmd) -> None:
        """Perform preparations for command run

        :param cmd: cliff Command object
        :returns: None

        """
        self.LOG.debug("prepare to run command %s", cmd.__class__.__name__)

    def clean_up(self, cmd, _, err) -> None:
        """Clean up after running the command.

        :param cmd: cliff Command object
        :param err: Error object
        :returns: None

        """
        self.LOG.debug("clean_up %s", cmd.__class__.__name__)
        if err:
            self.LOG.error("got an error: %s", err)


def main(argv=None) -> object:
    """Main script entry point.

    :param argv: Commandline arguments
    :returns: Result from invoked command

    """
    if argv is None:
        argv = sys.argv[1:]
    myapp = ScadiApp()
    return myapp.run(argv)


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['scadi']

package_data = \
{'': ['*']}

install_requires = \
['cliff>=3.10.0,<4.0.0']

entry_points = \
{'cliff.scadi': ['inline = scadi.inline:Inline'],
 'console_scripts': ['scadi = scadi.main:main']}

setup_kwargs = {
    'name': 'scadi',
    'version': '0.1.0',
    'description': 'SCAD Inliner: Roll up OpenSCAD includes into the main file for easy sharing.',
    'long_description': '=====\nscadi\n=====\n\nCommand-line tool for rolling up all includes into the main file of your model so that you can easily share it online.\n\nInstallation\n============\n\n::\n\n   pip3 install scadi\n\nUsage\n=====\n\n::\n\n   scadi inline ./my-model.scad\n\nThe above command will create a file called ``./inline-my-model.scad`` that can be shared on sites that have OpenSCAD customizers.\n',
    'author': 'Nascent Maker',
    'author_email': 'hello@nascentmaker.com',
    'maintainer': 'Nascent Maker',
    'maintainer_email': 'hello@nascentmaker.com',
    'url': 'https://nascentmaker.com/py/scadi',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
